/****************************************************************************
 * WangLandau: WangLandau simulation for RNA turner energy's density of state
 * Copyright (C) 2008-2009 Laboratoire de Recherche Informatique (LRI)
 * All Rights Reserved
 *
 *     This source code is distributed under the terms of the
 *     GNU General Public License.
 ****************************************************************************/

/*
 * rnaSecStr.c
 * RNA secondary structure functions
 * 2009/06/04 by Feng LOU & Peter CLOTE
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#include <time.h>
#include "../lib/rnaSecStr.h"

void error(char *s){
	fprintf(stderr,"%s\n",s);
	exit(1);
}
/*
 * functions check if the 2 letter can be base pair
 */
int guPair(char x, char y){
	return ( x=='G'&&y=='U' || x=='U'&&y=='G' );
}
int auPair(char x, char y){
	return ( x=='A'&&y=='U' || x=='U'&&y=='A' );
}
int cgPair(char x, char y){
	return ( x=='C'&&y=='G' || x=='G'&&y=='C' );
}
int a(int i, int j, char *s){
	if (i<strlen(s) && j<strlen(s)) {
		if (auPair(s[i],s[j]))
			return AUbond;
		else if (cgPair(s[i],s[j]))
			return CGbond;
		else if (guPair(s[i],s[j]))
			return GUbond;
		else return INF;
	}
	else
		error("Indices out of bounds for RNA sequence\n");
}
int basePair(char x, char y){
	return( auPair(x,y) || cgPair(x,y) || guPair(x,y) );
}
int watsonCrick(char x, char y){
	return( auPair(x,y) || cgPair(x,y) );
}
void printSecStrListAsString(char* secStr) {
	int n = strlen(secStr);
	secStr[n] = '\0';
	printf("%s\n", secStr);
}
/*
 * check the RNA sequence
 */
int validNucleotideString(char* rna) {
	int i, j;
	int n = strlen(rna);
	j = 1;
	for (i = 0; i < n; i++) {
		if (rna[i] != 'G' && rna[i] != 'C' && rna[i] != 'A' && rna[i] != 'U' && rna[i] != '&' && rna[i] != '.') {
			j = 0;
		}
	}
	return j;
}
int preValidSecStr(char* rna, char* secStr) {
	int m = strlen(rna);
	int n = strlen(secStr);
	if (m != n)
		return 0;
	int numLeftParen, numRightParen, numDots, i;
	numLeftParen = 0;
	numRightParen = 0;
	numDots = 0;
	for (i = 0; i < m; i++) {
		if (rna[i] == "(")
			numLeftParen++;
		else if (rna[i] == ")")
			numRightParen++;
		else if (rna[i] == ".")
			numDots++;
	}
	if (numLeftParen != numRightParen)
		return 0;
	if (numLeftParen + numRightParen + numDots != n)
		return 0;
	return 1;
}
/*
 * create an String for the empty secondary structure without base pair
 */
char* initialEmptySecStr(int n) {
	char* secStr = (char*) malloc((n + 1) * sizeof(char));
	int i;
	for (i = 0; i < n; i++) {
		secStr[i] = '.';
	}
	secStr[n] = '\0';
	return secStr;
}
char* getInitialSecStr(char* ssi) {
	int n = strlen(ssi);
	char* secStr = (char*) malloc((n + 1) * sizeof(char));

	int i;
	for (i = 0; i < n; i++) {
		secStr[i] = ssi[i];
	}
	secStr[n] = '\0';
	return secStr;
}
