/****************************************************************************
 * WangLandau: WangLandau simulation for RNA turner energy's density of state
 * Copyright (C) 2008-2009 Laboratoire de Recherche Informatique (LRI)
 * All Rights Reserved
 *
 *     This source code is distributed under the terms of the
 *     GNU General Public License.
 ****************************************************************************/

/*
 * struct.c
 * structure type definitions and functions
 * 2009/06/04 by Feng LOU & Peter CLOTE
 */
#include <stdio.h>
#include <stdlib.h>
#include "../lib/struct.h"

/*
 * print structure table
 */
void print_char(char* o) {
	printf("%s\n", o);
}
void print_int(int* o, int size) {
	int i;
	printf("[ ");
	for (i = 0; i < size; i++) {
		if (o[i] > 0)
			printf("%d: %d, ", i, o[i]);
	}
	printf("]\n");
}
void print_double(double* o, double init, int size) {
	int i;
	printf("[ ");
	for (i = 0; i < size; i++) {
		if (o[i] > init)
			printf("%d: %g, ", i, o[i]);
	}
	printf("]\n");
}
/*
 * return a new char* copy from original
 */
char* copyTo_char(char* o) {
	int size = strlen(o);
	char* r = (char*) malloc((size + 1) * sizeof(char));
	int i;
	for (i = 0; i < size; i++) {
		r[i] = o[i];
	}
	r[size] = '\0';
	return r;
}
/*
 * check if the distance between each base pair is > THRESHOLD
 */
int checkThreshold(char* secStr) {
	int i, j, k;
	int first = 0;
	int noSecStr = 1;
	int size = strlen(secStr);
	for (i = 0; i < size; i++) {
		if (secStr[i] == '(') {
			j = i;
			noSecStr = 0;
		}
		if (secStr[i] == ')' && first == 0) {
			k = i;
			first = 1;
			noSecStr = 0;
		}
	}
	if ((k - j) >= THRESHOLD || noSecStr == 1)
		return 1;
	else
		return 0;
}
/*
 * check if 2 strings is the same
 * return 1 if 2 String is same, else return 0
 */
int compare2String(char* s1, char* s2) {
	if (strlen(s1) != strlen(s2)) {
		printf("Problem in list of secondary structure!");
		exit(0);
	} else {
		int n = strlen(s1);
		int i;
		for (i = 0; i < n; i++) {
			if (s1[i] != s2[i])
				return 0;
		}
		return 1;
	}
}
/*
 * list functions **
 * create a new list
 */
list* init_list() {
	list* l = (list*) malloc(sizeof(list));
	l->first = NULL;
	l->size = 0;
	return l;
}
/*
 * return the x value at position 'pos' of list
 */
int get_list_x(list *l, int pos) {
	if (pos >= l->size) {
		printf("bad position, try another smaller position!");
		return -1;
	} else {
		item* tmp = l->first;
		int i = 0;
		while (i != pos) {
			tmp = tmp->next;
			i++;
		}
		return tmp->x;
	}
}
/*
 * return the y value at position 'pos' of list
 */
int get_list_y(list *l, int pos) {
	if (pos >= l->size) {
		printf("bad position, try another smaller position!");
		return -1;
	} else {
		item* tmp = l->first;
		int i = 0;
		while (i != pos) {
			tmp = tmp->next;
			i++;
		}
		return tmp->y;
	}
}
/*
 * set the x value at position 'pos' of list
 */
int set_list_x(list *l, int pos, int x) {
	if (pos >= l->size) {
		printf("bad position, try another smaller position!");
		return -1;
	} else {
		item* tmp = l->first;
		int i = 0;
		while (i != pos) {
			tmp = tmp->next;
			i++;
		}
		tmp->x = x;
	}
	return 1;
}
/*
 * set the y value at position 'pos' of list
 */
int set_list_y(list *l, int pos, int y) {
	if (pos >= l->size) {
		printf("bad position, try another smaller position!");
		return -1;
	} else {
		item* tmp = l->first;
		int i = 0;
		while (i != pos) {
			tmp = tmp->next;
			i++;
		}
		tmp->y = y;
	}
	return 1;
}
/*
 * add a new item at the end of list
 */
void add_list_item(list *l, int x, int y) {
	item* a = (struct item*) malloc(sizeof(struct item));
	if (x > y) {
		int tmp = x;
		x = y;
		y = tmp;
	}
	(a)->x = x;
	(a)->y = y;
	(a)->next = NULL;
	if (l->size > 0) {
		item *tmp = l->first;
		while (tmp->next != NULL) {
			tmp = tmp->next;
		}
		tmp->next = a;
	} else
		l->first = a;
	l->size++;
}
/*
 * remove the item at position 'pos' of list
 */
void remove_list_item(list* l, int pos) {
	if (l->size == 0) {
		printf("empty list, no item to remove. \n");
		exit(0);
	} else if (pos > l->size - 1) {
		printf("bad position, try another position smaller than %d.\n", pos);
		exit(0);
	} else if (pos == 0) {
		if (l->size == 1) {
			free(l->first);
			l->first = NULL;
			l->size = 0;
		} else {
			item* tmp = l->first->next;
			free(l->first);
			l->first = tmp;
			l->size--;
		}
	} else {
		item* tmp = l->first;
		int i = 0;
		while (i != pos - 1) {
			tmp = tmp->next;
			i++;
		}
		item* next_item = tmp->next->next;
		free(tmp->next);
		tmp->next = next_item;
		l->size--;
	}
}
/*
 *  create a new list copy from original list
 */
list* copy_list(list* l) {
	list* tmp = init_list();
	item* parser = l->first;
	while (parser != NULL) {
		add_list_item(tmp, parser->x, parser->y);
		parser = parser->next;
	}
	return tmp;
}
/*
 * print the x and y value of each item in list
 */
void print_list(list* l) {
	if (l->first == NULL) {
		printf("[]\n");
	} else {
		item* tmp = l->first;
		printf("[ ");
		while (tmp != NULL) {
			printf("(%d %d) ", tmp->x, tmp->y);
			tmp = tmp->next;
		}
		printf("]\n");
	}
}
/*
 * return all the base pair (x, y) from a Secondary Structure
 */
list* set_list(char* secStr) {
	list* tmp = init_list();
	int size = strlen(secStr);
	int k = 0;
	int s = 0;
	int left[size];
	// parser the secStr
	int i;
	for (i = 0; i < size; i++) {
		if (secStr[i] == '(') {
			left[k] = i;
			k = k + 1;
		} else if (secStr[i] == ')') {
			add_list_item(tmp, left[k - 1], i);
			k = k - 1;
			s = s + 1;
		}
	}
	tmp->size = s;
	return tmp;
}
/*
 * free memory used by list
 */
void free_list(list* l) {
	if (l->size == 0) {
		free(l);
	} else {
		item* parser = l->first;
		item* tmp;
		while (parser != NULL) {
			tmp = parser->next;
			l->first = tmp;
			free(parser);
			parser = tmp;
		}
		free(l);
	}
}
/*
 * create a new histogram with a new_size = 2 * old_size
 */
int* new_h(int* h, int size) {
	int* nh = (int*) malloc(2 * size * sizeof(int));
	int i;
	//sum of all histogram
	nh[2 * size - 1] = nh[size - 1];
	// copy old
	for (i = 0; i < size - 1; i++) {
		nh[i] = h[i];
	}
	// initialization new
	for (i = size - 1; i < 2 * size - 1; i++) {
		nh[i] = 0;
	}
	free(h);
	return nh;
}
/*
 * create a new density of state table with a new_size = 2 * old_size
 */
double* new_g(double* g, int size, double ds_init) {
	double* ng = (double*) malloc(2 * size * sizeof(double));
	int i;
	// copy old
	for (i = 0; i < size; i++) {
		ng[i] = g[i] / 2;
	}
	// initialization new
	for (i = size; i < 2 * size; i++) {
		ng[i] = 1.0 / (2 * size);
	}
	ds_init = ds_init / 2;
	free(g);
	return ng;
}
/*
 * create a new list of seconday structure with a new_size = 2 * old_size
 */
char** new_lss(char** lss, int size) {
	char** nlss;
	nlss = (char**) malloc(2 * size * sizeof(char*));
	// initialization size -> 2*size
	int i;
	for (i = 0; i < 2 * size; i++) {
		nlss[i] = NULL;
	}
	// copy old
	for (i = 0; i < size; i++) {
		if (lss[i] != NULL) {
			nlss[i] = copyTo_char(lss[i]);
		}
	}
	// free
	for (i = 0; i < size; i++) {
		free(lss[i]);
	}
	free(lss);
	return nlss;
}
/*
 * print the secStr energy and number value of each item in list_ss
 */
void print_list_ss(list_ss* l) {
	if (l->first == NULL) {
		printf("[]\n");
	} else {
		printf("\n\n%d \t different secondary structures\n\n", l->size);
		item_ss* tmp = l->first;
		while (tmp != NULL) {
			printf("[ %s \t %f \t %d ]\n", tmp->ss, tmp->energy, tmp->nb);
			tmp = tmp->next;
		}
	}
}
/*
 * free memory used by list_ss
 */
void free_list_ss(list_ss* lss) {
	item* parser = lss->first;
	item* tmp;
	while (parser != NULL) {
		tmp = parser->next;
		free(parser);
		parser = tmp;
	}
	free(lss);
}
/*
 * create a new list_ss
 */
list_ss* init_list_ss(void) {
	list_ss* l = (list_ss*) malloc(sizeof(list_ss));
	l->first = NULL;
	l->size = 0;
	return l;
}
