/****************************************************************************
 * WangLandau: WangLandau simulation for RNA turner energy's density of state
 * Copyright (C) 2008-2009 Laboratoire de Recherche Informatique (LRI)
 * All Rights Reserved
 *
 *     This source code is distributed under the terms of the
 *     GNU General Public License.
 ****************************************************************************/

/*
 * tools.c
 * some tools functions
 * 2009/06/04 by Feng LOU & Peter CLOTE
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "../lib/rnaSecStr.h"
#include "../lib/struct.h"

/*
 * return a list of base pair found in secStr
 */
list* getBasePairList(char* secStr) {
	char c1;
	int c2, i;
	int n = strlen(secStr);
	list* L = init_list();
	int* S = (int*) malloc(n * sizeof(int));
	int size = 0;
	// Parser secStr
	for (i = 0; i < n; i++) {
		c1 = secStr[i];
		if (c1 == '(') {
			S[size] = i;
			size++;
		} else if (c1 == ')') {
			if (size == 0) {
				//return -1;
			} else {
				c2 = S[size - 1];
				add_list_item(L, c2, i);
				size--;
				L->size++;
			}
		}
	}
	if (size != 0) {/*return 1;*/
	}
	return L;
}
/*
 * read the RNA sequence from FASTA file or command line
 */
char* getInputRNA(int flag, char* rnaStringOrFile, char* seqInfo) {
	char* rna;
	int i = 0;
	int j = 0;
	int n;

	if (flag == 1) {
		n = strlen(rnaStringOrFile);
		rna = (char*) malloc((n + 1) * sizeof(char));
		// upper case the RNA sequence and change letter T to U
		while (rnaStringOrFile[i]) {
			if (rnaStringOrFile[i] == 't' || rnaStringOrFile[i] == 'T') {
				rnaStringOrFile[i] = 'U';
			}
			rnaStringOrFile[i] = toupper(rnaStringOrFile[i]);
			rna[i] = rnaStringOrFile[i];
			i++;
		}
		rna[i] = '\0';
		sprintf(seqInfo, "> RNA 1");
		seqInfo[7] = '\0';
	} else if (flag == 2) {
		FILE *fp;
		char ch;
		rna = (char*) malloc(10000 * sizeof(char));
		if ((fp = fopen(rnaStringOrFile, "r")) == NULL) {
			printf("Cannot open file.\n");
			exit(1);
		}
		int m = 2;
		while ((ch = fgetc(fp)) != EOF) {
			if (ch == '>') {// check all item in L0 exist in L1 or not
				m = 1;
				seqInfo[j] = ch;
				j++;
			} else if (m == 1) {
				if (ch != '\n') {
					seqInfo[j] = ch;
					j++;
				} else {
					m = 0;
					seqInfo[j] = '\0';
				}
			} else if (m == 0) { // rna sequence line
				if (ch == 't' || ch == 'T') {
					ch == 'U';
				}
				rna[i] = toupper(ch);
				if (rna[i] != 'G' && rna[i] != 'C' && rna[i] != 'A' && rna[i]
						!= 'U') {
					rna[i] = '\0';
					m = 2;
					break;
				}
				i++;
			}
		}
		fclose(fp);
	}
	return rna;
}

/*
 * print the density of states for all energy turner with the bin of 0.1
 */
void printDensityOfStates(double* g, char** lss, int size, double energy_min,
		double energy_init) {
	printf("\n**************************************\n"
		"***** Turner energy distribution *****\n"
		"**************************************\n\n"
		"turner energy\trelative frequency\tsecondary structure\n");
	// calculate sum of density of state no empty
	int i, j, energy_t;
	double sum = 0.0;
	for (i = 0; i < size; i++) {
		if (g[i] > energy_init && lss[i] != NULL) {
			sum = sum + g[i];
		}
	}
	// print the normalized density of turner energy
	double energy, density;
	char* ss;
	for (i = 0; i < size; i++) {
		// energy got here is the inferior value of class
		energy_t = round((energy_min+0.1*i)*10);
		energy = energy_t / 10.0;
		density = g[i] / sum;
		ss = lss[i];
		if (g[i] > energy_init && lss[i] != NULL) {
			// density of turner energy
			printf("%5.1f\t\t%5.10g\t\t%s\n", energy, density, ss);
		}
	}
	printf("\n\n");
}
/*
 * print the density of states for all energy turner with the bin of 0.1 into file
 */
void printDensityOfStatesToFile(double* g, char** lss, int size,
		double energy_min, double energy_init, FILE* f) {
	// calculate sum of density of state no empty
	int i, j, energy_t;
	double sum = 0.0;
	for (i = 0; i < size; i++) {
		if (g[i] > energy_init && lss[i] != NULL) {
			sum = sum + g[i];
		}
	}
	// print the normalized density of turner energy
	double energy, density;
	char* ss;
	for (i = 0; i < size; i++) {
		// energy got here is the inferior value of class
		energy_t = round((energy_min+0.1*i)*10);
		energy = energy_t / 10.0;
		density = g[i] / sum;
		ss = lss[i];
		if (g[i] > energy_init && lss[i] != NULL) {
			// density of turner energy
			fprintf(f, "%5.1f\t\t%5.10g\t\t%s\n", energy, density, ss);
		}
	}
}
/*
 * check if base pair in position [i,j] can be added or not
 */
int canAddBasePair(int i, int j, int n, list* L) {
	int m = L->size;
	int k;
	if (i >= 0 && j > i && n > j) {
		for (k = 0; k < m; k++) {
			if ((get_list_x(L, k) == i && get_list_y(L, k) != j)
					|| (get_list_x(L, k) != i && get_list_y(L, k) == j)) {
				return 0;
			}
			if ((i < get_list_x(L, k) && get_list_x(L, k) < j && j
					< get_list_y(L, k)) || (get_list_x(L, k) < i && i
					< get_list_y(L, k) && get_list_y(L, k) < j)) {
				return 0;
			}
		}
		return 1;
	} else {
		return 0;
	}
}
/*
 * Usage information of Program
 */
void usage(void) {
	nrerror(
			"Usage:\n\n"
				"WangLandau -s RNA_Sequence | -f RNA_SEQUNECE_File | -n num_MC_steps | -c "
				"flatness_Constant | -e epsilon_value | -p [0|1|2] | -i sec_Strut_INI | -o output | "
				"-t temperature | -N total_num_MC_step \n\n "
				"-s: RNA nucleotide string (upper or lower case)\n "
				"-f: FASTA format file of RNA nucleotides\n "
				"-n: number of Monte Carlo steps, default is 100\n "
				"-c: flatness constant in (0,1), default is 0.8\n "
				"-e: epsilon value, default is 0.001\n "
				"-p: calculate the ensemble free diversity (1) or calculate the mean Boltzmann weighted distance (2) or no (0) \n "
				"-i: initial secondary structure\n "
				"-o: output file name, default is out\n "
				"-t: changer temperature, default is 37\n "
				"-N: total number of Monte Carlo step in program, default is none\n\n");
}
